@@ -5,8 +5,8 @@
 #define API_ENTRY \
    Smart_Data *sd = evas_object_smart_data_get(obj); \
    if ((!obj) || (!sd) || \
-          (evas_object_type_get(obj) && \
-              strcmp(evas_object_type_get(obj), SMART_NAME)))
+	  (evas_object_type_get(obj) && \
+	      strcmp(evas_object_type_get(obj), SMART_NAME)))
 #define INTERNAL_ENTRY \
    Smart_Data *sd = evas_object_smart_data_get(obj); \
    if (!sd) return;
@@ -14,7 +14,7 @@
 typedef struct _Smart_Data Smart_Data;
 
 struct _Smart_Data
-{ 
+{
    Evas_Object   *obj;
    const char    *type;
    Evas_Object   *parent_obj;
@@ -28,6 +28,7 @@ struct _Smart_Data
    void         (*activate_func) (Evas_Object *obj);
    void         (*disable_func) (Evas_Object *obj);
    void         (*theme_func) (Evas_Object *obj);
+   void         (*changed_func) (Evas_Object *obj);
    void         (*on_focus_func) (void *data, Evas_Object *obj);
    void          *on_focus_data;
    void         (*on_change_func) (void *data, Evas_Object *obj);
@@ -44,7 +45,7 @@ struct _Smart_Data
    unsigned char  child_can_focus : 1;
    unsigned char  focused : 1;
    unsigned char  disabled : 1;
-}; 
+};
 
 /* local subsystem functions */
 static void _smart_reconfigure(Smart_Data *sd);
@@ -57,6 +58,7 @@ static void _smart_hide(Evas_Object *obj);
 static void _smart_color_set(Evas_Object *obj, int r, int g, int b, int a);
 static void _smart_clip_set(Evas_Object *obj, Evas_Object * clip);
 static void _smart_clip_unset(Evas_Object *obj);
+static void _smart_calculate(Evas_Object *obj);
 static void _smart_init(void);
 
 /* local subsystem globals */
@@ -66,15 +68,13 @@ static void
 _sub_obj_del(void *data, Evas *e, Evas_Object *obj, void *event_info)
 {
    Smart_Data *sd = data;
+
    if (obj == sd->resize_obj)
-     {
-        sd->resize_obj = NULL;
-     }
-   else if (obj == sd->hover_obj) sd->hover_obj = NULL;
+     sd->resize_obj = NULL;
+   else if (obj == sd->hover_obj)
+     sd->hover_obj = NULL;
    else
-     {
-        sd->subobjs = eina_list_remove(sd->subobjs, obj);
-     }
+     sd->subobjs = eina_list_remove(sd->subobjs, obj);
    evas_object_smart_callback_call(sd->obj, "sub-object-del", obj);
 }
 
@@ -82,9 +82,19 @@ static void
 _sub_obj_mouse_down(void *data, Evas *e, Evas_Object *obj, void *event_info)
 {
    Smart_Data *sd = data;
-   Evas_Object *op, *op2;
-
-   elm_widget_focus_steal(sd->obj);
+   Evas_Object *o;
+   const char *s;
+   
+   o = obj;
+   do {
+      s = evas_object_type_get(o);
+      if (!s) return;
+      if (!strcmp(s, "elm_widget")) break;
+      o = evas_object_smart_parent_get(o);
+   } while (o);
+   if (!o) return;
+   if (!elm_widget_can_focus_get(o)) return;
+   elm_widget_focus_steal(o);
 }
 
 /* externally accessible functions */
@@ -137,6 +147,13 @@ elm_widget_theme_hook_set(Evas_Object *obj, void (*func) (Evas_Object *obj))
    sd->theme_func = func;
 }
 
+EAPI void
+elm_widget_changed_hook_set(Evas_Object *obj, void (*func) (Evas_Object *obj))
+{
+   API_ENTRY return;
+   sd->changed_func = func;
+}
+
 EAPI void
 elm_widget_theme(Evas_Object *obj)
 {
@@ -194,6 +211,7 @@ elm_widget_sub_object_add(Evas_Object *obj, Evas_Object *sobj)
 {
    API_ENTRY return;
    double scale, pscale = elm_widget_scale_get(sobj);
+
    sd->subobjs = eina_list_append(sd->subobjs, sobj);
    if (!sd->child_can_focus)
      {
@@ -204,7 +222,8 @@ elm_widget_sub_object_add(Evas_Object *obj, Evas_Object *sobj)
 	Smart_Data *sd2 = evas_object_smart_data_get(sobj);
 	if (sd2)
 	  {
-	     if (sd2->parent_obj) elm_widget_sub_object_del(sd2->parent_obj, sobj);
+	     if (sd2->parent_obj)
+               elm_widget_sub_object_del(sd2->parent_obj, sobj);
 	     sd2->parent_obj = obj;
 	  }
      }
@@ -228,7 +247,7 @@ elm_widget_sub_object_del(Evas_Object *obj, Evas_Object *sobj)
 	Smart_Data *sd2 = evas_object_smart_data_get(sobj);
 	if (sd2) sd2->parent_obj = NULL;
      }
-   evas_object_event_callback_del(sobj, EVAS_CALLBACK_DEL, _sub_obj_del);
+   evas_object_event_callback_del_full(sobj, EVAS_CALLBACK_DEL, _sub_obj_del, sd);
    evas_object_smart_callback_call(obj, "sub-object-del", sobj);
 }
 
@@ -238,27 +257,30 @@ elm_widget_resize_object_set(Evas_Object *obj, Evas_Object *sobj)
    API_ENTRY return;
    if (sd->resize_obj)
      {
-        if (!strcmp(evas_object_type_get(sd->resize_obj), SMART_NAME))
-          {
-             Smart_Data *sd2 = evas_object_smart_data_get(sd->resize_obj);
-             if (sd2) sd2->parent_obj = NULL;
-          }
-	evas_object_event_callback_del(sd->resize_obj, EVAS_CALLBACK_DEL, _sub_obj_del);
-	evas_object_event_callback_del(sd->resize_obj, EVAS_CALLBACK_MOUSE_DOWN, _sub_obj_mouse_down);
+	if (!strcmp(evas_object_type_get(sd->resize_obj), SMART_NAME))
+	  {
+	     Smart_Data *sd2 = evas_object_smart_data_get(sd->resize_obj);
+	     if (sd2) sd2->parent_obj = NULL;
+	  }
+	evas_object_event_callback_del_full(sd->resize_obj, EVAS_CALLBACK_DEL,
+           _sub_obj_del, sd);
+	evas_object_event_callback_del_full(sd->resize_obj, EVAS_CALLBACK_MOUSE_DOWN,
+           _sub_obj_mouse_down, sd);
 	evas_object_smart_member_del(sd->resize_obj);
      }
    sd->resize_obj = sobj;
    if (sd->resize_obj)
      {
-        if (!strcmp(evas_object_type_get(sd->resize_obj), SMART_NAME))
-          {
-             Smart_Data *sd2 = evas_object_smart_data_get(sd->resize_obj);
-             if (sd2) sd2->parent_obj = obj;
-          }
+	if (!strcmp(evas_object_type_get(sd->resize_obj), SMART_NAME))
+	  {
+	     Smart_Data *sd2 = evas_object_smart_data_get(sd->resize_obj);
+	     if (sd2) sd2->parent_obj = obj;
+	  }
 	evas_object_clip_set(sobj, evas_object_clip_get(obj));
 	evas_object_smart_member_add(sobj, obj);
 	evas_object_event_callback_add(sobj, EVAS_CALLBACK_DEL, _sub_obj_del, sd);
-	evas_object_event_callback_add(sobj, EVAS_CALLBACK_MOUSE_DOWN, _sub_obj_mouse_down, sd);
+	evas_object_event_callback_add(sobj, EVAS_CALLBACK_MOUSE_DOWN,
+                                       _sub_obj_mouse_down, sd);
 	_smart_reconfigure(sd);
 	evas_object_smart_callback_call(obj, "sub-object-add", sobj);
      }
@@ -270,7 +292,8 @@ elm_widget_hover_object_set(Evas_Object *obj, Evas_Object *sobj)
    API_ENTRY return;
    if (sd->hover_obj)
      {
-	evas_object_event_callback_del(sd->hover_obj, EVAS_CALLBACK_DEL, _sub_obj_del);
+	evas_object_event_callback_del_full(sd->hover_obj, EVAS_CALLBACK_DEL,
+           _sub_obj_del, sd);
      }
    sd->hover_obj = sobj;
    if (sd->hover_obj)
@@ -309,6 +332,7 @@ elm_widget_focused_object_get(const Evas_Object *obj)
    const Evas_Object *subobj;
    const Eina_List *l;
    API_ENTRY return NULL;
+
    if (!sd->focused) return NULL;
    EINA_LIST_FOREACH(sd->subobjs, l, subobj)
      {
@@ -331,7 +355,7 @@ elm_widget_focus_jump(Evas_Object *obj, int forward)
 {
    API_ENTRY return 0;
    if (!elm_widget_can_focus_get(obj)) return 0;
-       
+
    /* if it has a focus func its an end-point widget like a button */
    if (sd->focus_func)
      {
@@ -345,8 +369,8 @@ elm_widget_focus_jump(Evas_Object *obj, int forward)
    else
      {
 	int focus_next;
-        int noloop = 0;
-	
+	int noloop = 0;
+
 	focus_next = 0;
 	if (!sd->focused)
 	  {
@@ -359,53 +383,57 @@ elm_widget_focus_jump(Evas_Object *obj, int forward)
 	  {
 	     if (forward)
 	       {
-                  if (elm_widget_can_focus_get(sd->resize_obj))
-                    {
-                       if ((focus_next) &&
-                           (!elm_widget_disabled_get(sd->resize_obj)))
-                         {
-                            /* the previous focused item was unfocused - so focus
-                             * the next one (that can be focused) */
-                            if (elm_widget_focus_jump(sd->resize_obj, forward)) return 1;
-                            else noloop = 1;
-                         }
-                       else
-                         {
-                            if (elm_widget_focus_get(sd->resize_obj))
-                              {
-                                 /* jump to the next focused item or focus this item */
-                                 if (elm_widget_focus_jump(sd->resize_obj, forward)) return 1;
-                                 /* it returned 0 - it got to the last item and is past it */
-                                 focus_next = 1;
-                              }
-                         }
-                    }
-                  if (!noloop)
-                    {
+		  if (elm_widget_can_focus_get(sd->resize_obj))
+		    {
+		       if ((focus_next) &&
+			   (!elm_widget_disabled_get(sd->resize_obj)))
+			 {
+			    /* the previous focused item was unfocused - so focus
+			     * the next one (that can be focused) */
+			    if (elm_widget_focus_jump(sd->resize_obj, forward))
+                              return 1;
+			    else noloop = 1;
+			 }
+		       else
+			 {
+			    if (elm_widget_focus_get(sd->resize_obj))
+			      {
+				 /* jump to the next focused item or focus this item */
+				 if (elm_widget_focus_jump(sd->resize_obj, forward))
+                                   return 1;
+				 /* it returned 0 - it got to the last item and is past it */
+				 focus_next = 1;
+			      }
+			 }
+		    }
+		  if (!noloop)
+		    {
 		       const Eina_List *l;
 		       Evas_Object *child;
 		       EINA_LIST_FOREACH(sd->subobjs, l, child)
-                         {
-                            if (elm_widget_can_focus_get(child))
-                              {
-                                 if ((focus_next) &&
-                                     (!elm_widget_disabled_get(child)))
-                                   {
-                                      /* the previous focused item was unfocused - so focus
-                                       * the next one (that can be focused) */
-                                      if (elm_widget_focus_jump(child, forward)) return 1;
-                                      else break;
-                                   }
-                                 else
-                                   {
-                                      if (elm_widget_focus_get(child))
-                                        {
-                                           /* jump to the next focused item or focus this item */
-                                           if (elm_widget_focus_jump(child, forward)) return 1;
-                                           /* it returned 0 - it got to the last item and is past it */
-                                           focus_next = 1;
-                                        }
-                                   }
+			 {
+			    if (elm_widget_can_focus_get(child))
+			      {
+				 if ((focus_next) &&
+				     (!elm_widget_disabled_get(child)))
+				   {
+				      /* the previous focused item was unfocused - so focus
+				       * the next one (that can be focused) */
+				      if (elm_widget_focus_jump(child, forward))
+                                        return 1;
+				      else break;
+				   }
+				 else
+				   {
+				      if (elm_widget_focus_get(child))
+					{
+					   /* jump to the next focused item or focus this item */
+					   if (elm_widget_focus_jump(child, forward))
+                                             return 1;
+					   /* it returned 0 - it got to the last item and is past it */
+					   focus_next = 1;
+					}
+				   }
 			      }
 			 }
 		    }
@@ -424,7 +452,8 @@ elm_widget_focus_jump(Evas_Object *obj, int forward)
 			      {
 				 /* the previous focused item was unfocused - so focus
 				  * the next one (that can be focused) */
-				 if (elm_widget_focus_jump(child, forward)) return 1;
+				 if (elm_widget_focus_jump(child, forward))
+                                   return 1;
 				 else break;
 			      }
 			    else
@@ -432,15 +461,16 @@ elm_widget_focus_jump(Evas_Object *obj, int forward)
 				 if (elm_widget_focus_get(child))
 				   {
 				      /* jump to the next focused item or focus this item */
-				      if (elm_widget_focus_jump(child, forward)) return 1;
+				      if (elm_widget_focus_jump(child, forward))
+                                        return 1;
 				      /* it returned 0 - it got to the last item and is past it */
 				      focus_next = 1;
 				   }
 			      }
 			 }
 		    }
-                  if (!l)
-                    {
+		  if (!l)
+		    {
 		       if (elm_widget_can_focus_get(sd->resize_obj))
 			 {
 			    if ((focus_next) &&
@@ -448,28 +478,30 @@ elm_widget_focus_jump(Evas_Object *obj, int forward)
 			      {
 				 /* the previous focused item was unfocused - so focus
 				  * the next one (that can be focused) */
-				 if (elm_widget_focus_jump(sd->resize_obj, forward)) return 1;
+				 if (elm_widget_focus_jump(sd->resize_obj, forward))
+                                   return 1;
 			      }
 			    else
 			      {
 				 if (elm_widget_focus_get(sd->resize_obj))
 				   {
 				      /* jump to the next focused item or focus this item */
-				      if (elm_widget_focus_jump(sd->resize_obj, forward)) return 1;
+				      if (elm_widget_focus_jump(sd->resize_obj, forward))
+                                        return 1;
 				      /* it returned 0 - it got to the last item and is past it */
 				      focus_next = 1;
 				   }
 			      }
 			 }
-                    }
+		    }
 	       }
 	  }
      }
    /* no next item can be focused */
    if (sd->focused)
      {
-        sd->focused = 0;
-        if (sd->on_focus_func) sd->on_focus_func(sd->on_focus_data, obj);
+	sd->focused = 0;
+	if (sd->on_focus_func) sd->on_focus_func(sd->on_focus_data, obj);
      }
    return 0;
 }
@@ -492,25 +524,25 @@ elm_widget_focus_set(Evas_Object *obj, int first)
      {
 	if (first)
 	  {
-             if ((elm_widget_can_focus_get(sd->resize_obj)) &&
-                 (!elm_widget_disabled_get(sd->resize_obj)))
-               {
-                  elm_widget_focus_set(sd->resize_obj, first);
-               }
-             else
-               {
+	     if ((elm_widget_can_focus_get(sd->resize_obj)) &&
+		 (!elm_widget_disabled_get(sd->resize_obj)))
+	       {
+		  elm_widget_focus_set(sd->resize_obj, first);
+	       }
+	     else
+	       {
 		  const Eina_List *l;
 		  Evas_Object *child;
 		  EINA_LIST_FOREACH(sd->subobjs, l, child)
-                    {
-                       if ((elm_widget_can_focus_get(child)) &&
-                           (!elm_widget_disabled_get(child)))
-                         {
-                            elm_widget_focus_set(child, first);
-                            break;
-                         }
-                    }
-               }
+		    {
+		       if ((elm_widget_can_focus_get(child)) &&
+			   (!elm_widget_disabled_get(child)))
+			 {
+			    elm_widget_focus_set(child, first);
+			    break;
+			 }
+		    }
+	       }
 	  }
 	else
 	  {
@@ -525,14 +557,14 @@ elm_widget_focus_set(Evas_Object *obj, int first)
 		       break;
 		    }
 	       }
-             if (!l)
-               {
-                  if ((elm_widget_can_focus_get(sd->resize_obj)) &&
-                      (!elm_widget_disabled_get(sd->resize_obj)))
-                    {
-                       elm_widget_focus_set(sd->resize_obj, first);
-                    }
-               }
+	     if (!l)
+	       {
+		  if ((elm_widget_can_focus_get(sd->resize_obj)) &&
+		      (!elm_widget_disabled_get(sd->resize_obj)))
+		    {
+		       elm_widget_focus_set(sd->resize_obj, first);
+		    }
+	       }
 	  }
      }
 }
@@ -550,21 +582,19 @@ elm_widget_focused_object_clear(Evas_Object *obj)
    API_ENTRY return;
    if (!sd->focused) return;
    if (elm_widget_focus_get(sd->resize_obj))
-     {
-        elm_widget_focused_object_clear(sd->resize_obj);
-     }
+     elm_widget_focused_object_clear(sd->resize_obj);
    else
      {
 	const Eina_List *l;
 	Evas_Object *child;
 	EINA_LIST_FOREACH(sd->subobjs, l, child)
-          {
-             if (elm_widget_focus_get(child))
-               {
-                  elm_widget_focused_object_clear(child);
-                  break;
-               }
-          }
+	  {
+	     if (elm_widget_focus_get(child))
+	       {
+		  elm_widget_focused_object_clear(child);
+		  break;
+	       }
+	  }
      }
    sd->focused = 0;
    if (sd->on_focus_func) sd->on_focus_func(sd->on_focus_data, obj);
@@ -576,6 +606,7 @@ _elm_widget_parent_focus(Evas_Object *obj)
 {
    API_ENTRY return;
    Evas_Object *o = elm_widget_parent_get(obj);
+
    if (sd->focused) return;
    if (o) _elm_widget_parent_focus(o);
    sd->focused = 1;
@@ -588,6 +619,7 @@ elm_widget_focus_steal(Evas_Object *obj)
 {
    Evas_Object *parent, *o;
    API_ENTRY return;
+
    if (sd->focused) return;
    if (sd->disabled) return;
    parent = obj;
@@ -595,31 +627,33 @@ elm_widget_focus_steal(Evas_Object *obj)
      {
 	o = elm_widget_parent_get(parent);
 	if (!o) break;
-        sd = evas_object_smart_data_get(o);
-        if (sd->focused) break;
+	sd = evas_object_smart_data_get(o);
+	if (sd->focused) break;
 	parent = o;
      }
    if (!elm_widget_parent_get(parent))
      elm_widget_focused_object_clear(parent);
    else
      {
-        parent = elm_widget_parent_get(parent);
-        sd = evas_object_smart_data_get(parent);
-        if (elm_widget_focus_get(sd->resize_obj))
-          elm_widget_focused_object_clear(sd->resize_obj);
-        else
+	parent = elm_widget_parent_get(parent);
+	sd = evas_object_smart_data_get(parent);
+	if (elm_widget_focus_get(sd->resize_obj))
           {
+             elm_widget_focused_object_clear(sd->resize_obj);
+          }
+	else
+	  {
 	     const Eina_List *l;
 	     Evas_Object *child;
 	     EINA_LIST_FOREACH(sd->subobjs, l, child)
-               {
-                  if (elm_widget_focus_get(child))
-                    {
-                       elm_widget_focused_object_clear(child);
-                       break;
-                    }
-               }
-          }
+	       {
+		  if (elm_widget_focus_get(child))
+		    {
+		       elm_widget_focused_object_clear(child);
+		       break;
+		    }
+	       }
+	  }
      }
    _elm_widget_parent_focus(obj);
    return;
@@ -645,16 +679,16 @@ EAPI void
 elm_widget_disabled_set(Evas_Object *obj, int disabled)
 {
    API_ENTRY return;
-   if (((sd->disabled) && (disabled)) ||
-       ((!sd->disabled) && (!disabled))) return;
+
+   if (sd->disabled == disabled) return;
    sd->disabled = disabled;
    if (sd->focused)
      {
 	Evas_Object *o, *parent;
 
 	parent = obj;
-        for (;;)
-          {
+	for (;;)
+	  {
 	     o = elm_widget_parent_get(parent);
 	     if (!o) break;
 	     parent = o;
@@ -680,7 +714,8 @@ elm_widget_show_region_set(Evas_Object *obj, Evas_Coord x, Evas_Coord y, Evas_Co
    sd->ry = y;
    sd->rw = w;
    sd->rh = h;
-   if (sd->on_show_region_func) sd->on_show_region_func(sd->on_show_region_data, obj);
+   if (sd->on_show_region_func)
+     sd->on_show_region_func(sd->on_show_region_data, obj);
 }
 
 EAPI void
@@ -701,6 +736,7 @@ elm_widget_scroll_hold_push(Evas_Object *obj)
    if (sd->scroll_hold == 1)
      evas_object_smart_callback_call(obj, "scroll-hold-on", obj);
    if (sd->parent_obj) elm_widget_scroll_hold_push(sd->parent_obj);
+   // FIXME: on delete/reparent hold pop
 }
 
 EAPI void
@@ -729,6 +765,7 @@ elm_widget_scroll_freeze_push(Evas_Object *obj)
    if (sd->scroll_freeze == 1)
      evas_object_smart_callback_call(obj, "scroll-freeze-on", obj);
    if (sd->parent_obj) elm_widget_scroll_freeze_push(sd->parent_obj);
+   // FIXME: on delete/reparent freeze pop
 }
 
 EAPI void
@@ -756,8 +793,8 @@ elm_widget_scale_set(Evas_Object *obj, double scale)
    if (scale <= 0.0) scale = 0.0;
    if (sd->scale != scale)
      {
-        sd->scale = scale;
-        elm_widget_theme(obj);
+	sd->scale = scale;
+	elm_widget_theme(obj);
      }
 }
 
@@ -770,7 +807,7 @@ elm_widget_scale_get(const Evas_Object *obj)
 	if (sd->parent_obj)
 	  return elm_widget_scale_get(sd->parent_obj);
 	else
-	  return elm_scale_get();
+	  return 1.0;
      }
    return sd->scale;
 }
@@ -780,6 +817,7 @@ elm_widget_style_set(Evas_Object *obj, const char *style)
 {
    const char *old;
    API_ENTRY return;
+
    old = sd->style;
    sd->style = eina_stringshare_add(style);
    if (old) eina_stringshare_del(old);
@@ -832,7 +870,7 @@ static void
 _smart_add(Evas_Object *obj)
 {
    Smart_Data *sd;
-   
+
    sd = calloc(1, sizeof(Smart_Data));
    if (!sd) return;
    sd->obj = obj;
@@ -848,29 +886,29 @@ static void
 _smart_del(Evas_Object *obj)
 {
    Evas_Object *sobj;
-   
+
    INTERNAL_ENTRY;
    if (sd->del_pre_func) sd->del_pre_func(obj);
    if (sd->resize_obj)
      {
-        sobj = sd->resize_obj;
+	sobj = sd->resize_obj;
 	sd->resize_obj = NULL;
-	evas_object_event_callback_del(sobj, EVAS_CALLBACK_DEL, _sub_obj_del);
-        evas_object_smart_callback_call(sd->obj, "sub-object-del", sd->resize_obj);
+	evas_object_event_callback_del_full(sobj, EVAS_CALLBACK_DEL, _sub_obj_del, sd);
+	evas_object_smart_callback_call(sd->obj, "sub-object-del", sd->resize_obj);
 	evas_object_del(sobj);
      }
    if (sd->hover_obj)
      {
-        sobj = sd->resize_obj;
+	sobj = sd->hover_obj;
 	sd->hover_obj = NULL;
-	evas_object_event_callback_del(sobj, EVAS_CALLBACK_DEL, _sub_obj_del);
-        evas_object_smart_callback_call(sd->obj, "sub-object-del", sobj);
+	evas_object_event_callback_del_full(sobj, EVAS_CALLBACK_DEL, _sub_obj_del, sd);
+	evas_object_smart_callback_call(sd->obj, "sub-object-del", sobj);
 	evas_object_del(sobj);
      }
    EINA_LIST_FREE(sd->subobjs, sobj)
      {
-	evas_object_event_callback_del(sobj, EVAS_CALLBACK_DEL, _sub_obj_del);
-        evas_object_smart_callback_call(sd->obj, "sub-object-del", sobj);
+	evas_object_event_callback_del_full(sobj, EVAS_CALLBACK_DEL, _sub_obj_del, sd);
+	evas_object_smart_callback_call(sd->obj, "sub-object-del", sobj);
 	evas_object_del(sobj);
      }
    if (sd->del_func) sd->del_func(obj);
@@ -930,7 +968,14 @@ _smart_clip_unset(Evas_Object *obj)
 {
    INTERNAL_ENTRY;
    evas_object_clip_unset(sd->resize_obj);
-}  
+}
+
+static void
+_smart_calculate(Evas_Object *obj)
+{
+   INTERNAL_ENTRY;
+   if (sd->changed_func) sd->changed_func(obj);
+}
 
 /* never need to touch this */
 
@@ -944,7 +989,7 @@ _smart_init(void)
 	     SMART_NAME,
 	       EVAS_SMART_CLASS_VERSION,
 	       _smart_add,
-	       _smart_del, 
+	       _smart_del,
 	       _smart_move,
 	       _smart_resize,
 	       _smart_show,
@@ -952,11 +997,45 @@ _smart_init(void)
 	       _smart_color_set,
 	       _smart_clip_set,
 	       _smart_clip_unset,
-	       NULL,
+	       _smart_calculate,
 	       NULL,
 	       NULL,
 	       NULL
 	  };
-        _e_smart = evas_smart_class_new(&sc);
+	_e_smart = evas_smart_class_new(&sc);
      }
 }
+
+
+Eina_List *
+_stringlist_get(const char *str)
+{
+   Eina_List *list = NULL;
+   const char *s, *b;
+   if (!str) return NULL;
+   for (b = s = str; 1; s++)
+     {
+	if ((*s == ' ') || (*s == 0))
+	  {
+	     char *t = malloc(s - b + 1);
+	     if (t)
+	       {
+		  strncpy(t, b, s - b);
+		  t[s - b] = 0;
+		  list = eina_list_append(list, eina_stringshare_add(t));
+		  free(t);
+	       }
+	     b = s + 1;
+	  }
+	if (*s == 0) break;
+     }
+   return list;
+}
+
+void
+_stringlist_free(Eina_List *list)
+{
+   const char *s;
+   EINA_LIST_FREE(list, s)
+     eina_stringshare_del(s);
+}
